/***************************************************************************
 *   Copyright (C) 2005 by Andrew Krause                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "autocomplete.h"

bool autocomplete_check_char (char ch)
{
  if (isalnum(ch) || ch == '_' || ch == '-')
    return true;
  else
    return false;
}

bool autocomplete_check_key (GdkEventKey event)
{
  guint key = event.keyval;
  
  if (key == GDK_minus)
    return true;
  else if (key == GDK_underscore)
    return true;
  else if (key >= GDK_0 && key <= GDK_9)
    return true;
  else if (key >= GDK_A && key <= GDK_Z)
    return true;
  else if (key >= GDK_a && key <= GDK_z)
    return true;
  else
    return false;
}

void autocomplete_hide_popup (gpointer data)
{
  NotebookManager *ptr = (NotebookManager*) data;
  
  if (GTK_WIDGET_VISIBLE(ptr->ac.autocompletewin))
    gtk_widget_hide_all (GTK_WIDGET(ptr->ac.autocompletewin));
}

bool autocomplete_insert_word (gpointer data)
{
  NotebookManager *ptr = (NotebookManager*) data;

  if (GTK_WIDGET_VISIBLE(ptr->ac.autocompletewin))
  {
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman_get_textbuffer());
    GtkTextIter begin, end;
  
    gtk_text_buffer_get_iter_at_mark (buffer, &begin, gtk_text_buffer_get_insert (buffer));
    end = begin;
  
    gtk_text_iter_backward_char (&begin);
    while (autocomplete_check_char(gtk_text_iter_get_char(&begin)))
      gtk_text_iter_backward_char (&begin);
    gtk_text_iter_forward_char (&begin);
  
    string slice = gtk_text_buffer_get_slice (buffer, &begin, &end, false);
    
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ptr->ac.autocomplete));
    GtkTreeModel *model;
    GtkTreeIter iter;
    string text;
    gchar *temp;

    gtk_tree_selection_get_selected(selection, &model, &iter);
    gtk_tree_model_get(model, &iter, 0, &temp, -1);
    text = temp;
    text.erase (0, slice.length());
    gtk_text_buffer_insert_at_cursor (buffer, text.c_str(), -1);    
    autocomplete_hide_popup (data);
    return true;
  }
  
  return false;
}

void autocomplete_populate (gpointer data,
                            vector<string> choices)
{
  NotebookManager *ptr = (NotebookManager*) data;
  
  GtkTreeModel *model;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  
  if (gtk_tree_view_get_model(GTK_TREE_VIEW(ptr->ac.autocomplete)))
        gtk_tree_view_remove_column(GTK_TREE_VIEW(ptr->ac.autocomplete),
            gtk_tree_view_get_column(GTK_TREE_VIEW(ptr->ac.autocomplete), 0));   
  
  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
  gtk_tree_view_set_model (GTK_TREE_VIEW(ptr->ac.autocomplete), model);
  
  /* Append a row and fill in some data */
  for (unsigned int i=0; i < choices.size(); i++)
  {
    gtk_list_store_append (GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, choices.at(i).c_str(), -1);
  }
  g_object_unref(model);
  
  renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Choices", renderer, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(ptr->ac.autocomplete), column);
  
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(ptr->ac.autocomplete));
  gtk_tree_selection_select_path (selection, gtk_tree_path_new_from_string("0"));
}

void autocomplete_position (gpointer data, int winx, int winy)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  GtkTextIter iter;
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman->noteman_get_textbuffer());
  GtkTextLayout *layout = GTK_TEXT_VIEW(ptr->noteman->noteman_get_textview())->layout;
  GdkRectangle rect;
  gint x, y, truex, truey, fontsize = atol(ptr->env->get_font_size().c_str()),
       spacing = fontsize / 4;
  
  // Get the mark & location of the cursor
  gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
                  gtk_text_buffer_get_mark (buffer, "insert"));
  gtk_text_iter_backward_char (&iter);
  while (autocomplete_check_char(gtk_text_iter_get_char(&iter)))
    gtk_text_iter_backward_char (&iter);
  gtk_text_iter_forward_char (&iter);
  gtk_text_layout_get_cursor_locations (layout, &iter, &rect, NULL);
  
  // Change the cursor coordinates to window coordinates
  gtk_text_view_buffer_to_window_coords (GTK_TEXT_VIEW(ptr->noteman->noteman_get_textview()), 
                                         GTK_TEXT_WINDOW_WIDGET, rect.x, rect.y, &truex, &truey);
  
  // Add together the window position, buffer coordinates, allocation and
  // (font size, titlebar spacing and spacing based on the font)
  x = truex + ptr->noteman->noteman_get_textview()->allocation.x + winx;
  y = truey + ptr->noteman->noteman_get_textview()->allocation.y + winy + fontsize + 30 + spacing;

  // Move the window into position
  gtk_window_move (GTK_WINDOW(ptr->noteman->ac.autocompletewin), x, y);
}

void autocomplete_scroll_view (gpointer data,
                               guint key)
{
  NotebookManager *ptr = (NotebookManager*) data;
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ptr->ac.autocomplete));
  GtkTreeModel *model;
  GtkTreeIter iter;
  
  gtk_tree_selection_get_selected(selection, &model, &iter);
  string path_str = gtk_tree_model_get_string_from_iter (model, &iter);
  GtkTreePath *path = gtk_tree_path_new_from_string (path_str.c_str());
  
  // Scroll up
  if (key == GDK_Down || key == GDK_KP_Down)
  {
    gtk_tree_path_next (path);
    gtk_tree_selection_select_path (selection, path);
  }
  // Scroll down
  else
  {
    if (gtk_tree_path_prev (path))
      gtk_tree_selection_select_path (selection, path);
  }
}

void autocomplete_show_popup (gpointer data, gchar ch)
{
  NotebookManager *ptr = (NotebookManager*) data;
  
  int length = 0;
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman_get_textbuffer());
  GtkTextIter begin, end;
  
  gtk_text_buffer_get_iter_at_mark (buffer, &begin, gtk_text_buffer_get_insert (buffer));
  end = begin;
  
  gtk_text_iter_backward_char (&begin);
  while (autocomplete_check_char(gtk_text_iter_get_char(&begin)))
  {
    gtk_text_iter_backward_char (&begin);
    length++;
  }
  gtk_text_iter_forward_char (&begin);
  
  string slice = gtk_text_buffer_get_slice (buffer, &begin, &end, false);
  slice += ch;

  if (slice.length() > 2)
  {
    vector<string> choices = ptr->al.autolist_get_matches (slice);
  
    if (choices.size() > 0)
    {
      autocomplete_populate (data, choices); 
  
      if (!GTK_WIDGET_VISIBLE(ptr->ac.autocompletewin))
        gtk_widget_show_all (GTK_WIDGET(ptr->ac.autocompletewin));
    }
    else
      autocomplete_hide_popup(data);
  }
  else
    autocomplete_hide_popup(data);
}

// Handle the key press and button press events
bool on_textview_keypress (GtkWidget *widget,
                           GdkEvent *ev,
                           gpointer data)
{
  NotebookManager *ptr = (NotebookManager*) data;

  if (ev->type == GDK_KEY_PRESS)
  {
    GdkEventKey event = ev->key;
    
    ptr->noteman_mark_current_tab_modified(true);
    
    if (autocomplete_check_key(event))
    {
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman_get_textbuffer());
      GtkTextIter cursor;
      gtk_text_buffer_get_iter_at_mark (buffer, &cursor,
                      gtk_text_buffer_get_insert(buffer));
      
      gchar *chs = gdk_keyval_name (event.keyval), ch;
      if (!strcmp(chs, "underscore")) ch = '_';
      else if (!strcmp(chs, "minus")) ch = '-';
      else ch = chs[0];
      
      if (gtk_text_iter_get_offset(&cursor) >= 5)
        autocomplete_show_popup (data, ch);
    }
    else if (event.keyval == GDK_Shift_L || event.keyval == GDK_Shift_R)
      return false;
    else if ((event.keyval == GDK_Up || event.keyval == GDK_Down ||
             event.keyval == GDK_KP_Up || event.keyval == GDK_KP_Down) &&
             GTK_WIDGET_VISIBLE(ptr->ac.autocompletewin))
    {
      autocomplete_scroll_view (data, event.keyval);
      return true;
    }
    else if (event.keyval == GDK_KP_Enter || event.keyval == GDK_Return)
      return autocomplete_insert_word(data);
    else
      autocomplete_hide_popup (data);
  }
  else if (ev->type == GDK_BUTTON_PRESS)
    autocomplete_hide_popup (data);
  
  return false;
}
