/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "autolist.h"

void AutoList::autolist_add_file (string fn)
{
  int location = autolist_find_file(fn);
  if (location != -1)
  {
    unsigned int i, j, k;
    for (i=0; i < words.size(); i++)
      for (j=0; j < words.at(i).files.size(); j++)
        if (words.at(i).files.at(j) == location)
        {
          vector<int>::iterator itr = words.at(i).files.begin();
          for (k=0; k < j; k++)
            itr++;
          words.at(i).files.erase(itr);
          break;
        }
  }
  
  AutoListFile file;
  file.filename = fn;
  files.push_back(file);
  
  autolist_parse_file (files.size()-1);
}

void AutoList::autolist_add_word (string word, int file)
{
  int fword = autolist_find_word (word);
  
  if (fword != -1)
    for (unsigned int i=0; i < words.at(fword).files.size(); i++)
      if (words.at(fword).files.at(i) == file)
        return;
  
  if (fword != -1)
    words.at(fword).files.push_back(file);
  else
  {
    AutoListWord wd;
    wd.word = word;
    wd.files.push_back(file);
    
    if (words.size() > 0)
    {
      unsigned int count=0;
      vector<AutoListWord>::iterator iter = words.begin();
    
      while (words.at(count).word < wd.word)
      {
        count++;
        iter++;
        
        if (words.size() == count)
          break;
      }

      if (words.size() == count)
        words.push_back (wd);
      else
        words.insert (iter, wd);
    }
    else
      words.push_back (wd);
  }
}

bool AutoList::autolist_check_char (char ch)
{
  if (isalnum(ch) || ch == '_' || ch == '-')
    return true;
  else
    return false;
}

void AutoList::autolist_clean_all ()
{
  while (files.size() > 0)
    files.pop_back();
  
  while (words.size() > 0)
    words.pop_back();
}

int AutoList::autolist_find_file (string file)
{
  for (unsigned int i=0; i < files.size(); i++)
    if (files.at(i).filename == file)
      return i;
  
  return -1;
}

int AutoList::autolist_find_word (string word)
{
  int high = words.size(), low = -1, mid;
  
  if (high > 0)
  {
    while ((high - low) > 1)
    {
      mid = (high + low) / 2;
      if (words.at(mid).word > word)
        high = mid;
      else
        low = mid;
    }
    
    if (low == -1 || words.at(low).word != word)
      return -1;
    else
      return low;
  }
  
  return -1;
}

vector<string> AutoList::autolist_get_matches (string match)
{
  int len = match.length(), location = -1;
  vector<string> choices;

  for (unsigned int i = 0; i < words.size(); i++)
    if (words.at(i).word.substr(0, len) == match)
    {
      location = i;
      break;
    }
  
  if (location != -1)
    for (int w = location; w < location + 3; w++)
      if (w < (signed int)words.size())
        if (words.at(w).word.substr(0, len) == match &&
            words.at(w).word != match)
          choices.push_back(words.at(w).word);
    
  return choices;
}

void AutoList::autolist_parse_file (int location)
{
  char buffer[256];
  string text, word;
  ifstream fin(files.at(location).filename.c_str());
  
  while (!fin.eof())
  {
    fin.getline (buffer, 255);
    text += string(buffer);
  }
  
  while (text.length() > 0)
  {
    if (autolist_check_char(text[0]))
      word += text[0];
    else if (!word.empty() && word.length() >= 4 &&
             (isalpha(word[0]) || word[0] == '_'))
    {
      autolist_add_word (word, location);
      word.clear();
    }
    else
      word.clear();
    
    text.erase(0, 1);
  }
}

void AutoList::autolist_parse_files (vector<string> fns)
{
  for (unsigned int i=0; i < fns.size(); i++)
    autolist_add_file (fns.at(i));
}
