/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "browserfiles.h"

// Default constructor for the BrowserFiles class. This declares the TreeView
// object & then adds it to the scrolled window. It also makes sure that the
// header for the TreeView is not visible
BrowserFiles::BrowserFiles ()
{
  treeview = gtk_tree_view_new();
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), false);
  
  swin = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), treeview);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), 
                                 GTK_POLICY_AUTOMATIC, 
                                 GTK_POLICY_ALWAYS);
}

// Add the project files to the GtkTreeView. This function receives a
// pointer to an instance of the project settings class.
void BrowserFiles::populate (ProjectSettings *project)
{
  int start=0;
  string text = project->get_source_files(), file;
  GtkTreeIter row_sources, row_headers, row_others, child;
  
  if (gtk_tree_view_get_model(GTK_TREE_VIEW(treeview)))
        gtk_tree_view_remove_column(GTK_TREE_VIEW(treeview),
            gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), 0));
  
  // Create project files tree view
  GtkTreeModel *model = GTK_TREE_MODEL(gtk_tree_store_new(1, G_TYPE_STRING));
  gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), model);
  
  // Add the TreeView's view columns:
  GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes("Project Files",
                              renderer, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);

  // Fill the TreeView's model
  gtk_tree_store_append(GTK_TREE_STORE(model), &row_sources, NULL);
  gtk_tree_store_set(GTK_TREE_STORE(model), &row_sources, 0, "Sources", -1);
  gtk_tree_store_append(GTK_TREE_STORE(model), &row_headers, NULL);
  gtk_tree_store_set(GTK_TREE_STORE(model), &row_headers, 0, "Headers", -1);
  gtk_tree_store_append(GTK_TREE_STORE(model), &row_others, NULL);
  gtk_tree_store_set(GTK_TREE_STORE(model), &row_others, 0, "Other Files", -1);

  // Append the sources
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      gtk_tree_store_append(GTK_TREE_STORE(model), &child, &row_sources);
      gtk_tree_store_set(GTK_TREE_STORE(model), &child, 0, 
                         text.substr(start, i-start).c_str(), -1);
      start = i+1;
    }
  }
  
  // Append the header files
  start=0;
  text = project->get_header_files();
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      gtk_tree_store_append(GTK_TREE_STORE(model), &child, &row_headers);
      gtk_tree_store_set(GTK_TREE_STORE(model), &child, 0, 
                         text.substr(start, i-start).c_str(), -1);
      start = i+1;
    }
  }
  
  // Append the other files
  start=0;
  text = project->get_other_files();
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      gtk_tree_store_append(GTK_TREE_STORE(model), &child, &row_others);
      gtk_tree_store_set(GTK_TREE_STORE(model), &child, 0, 
                         text.substr(start, i-start).c_str(), -1);
      start = i+1;
    }
  }
    
  // Expand all of the tree to show files
  gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  gtk_tree_model_get_iter(model, &child, gtk_tree_path_new_from_string("0"));
  gtk_tree_selection_select_iter(selection, &child);
}

// Get the GtkTreeView object of the project file browser.
GtkWidget* BrowserFiles::get_treeview()
{ return treeview;
}

GtkWidget* BrowserFiles::get_swin()
{ return swin;
}
