/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "environment.h"

// This constructor initializes all of the widgets and sets their properties. It
// then creates and shows the dialog. It receives the environment settings object
EnvironmentDialog::EnvironmentDialog (EnvironmentSettings *env) 
{
  fontname = string(env->get_font_face()) + " " + string(env->get_font_size());
  
  // Create objects for dialog
  initialize_objects();
  
  // Create the General editor tab
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(tab_size), env->get_tabsize());
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(asave_minutes), env->get_autosave());
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(undo_actions), env->get_undo());
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(replace_tabs), env->get_replace_tabs());
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(load_on_launch), env->get_autoload());
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(autoindent), env->get_auto_indent());
  
  // Create the View tab
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(text_wrap), env->get_wrap());
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(split_words), env->get_wrapsplit());
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(bracket), env->get_brackets());
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(right_margin), env->get_rightmargin());
  
  // Create the Fonts & Colors tab
  string fontnametemp = string(env->get_font_face() + " " + env->get_font_size());
  GdkColor tmp = { 0, 0, 0, 0, },
           *errorcolor = gdk_color_copy(&tmp),
           *warningcolor = gdk_color_copy(&tmp);
  gdk_color_parse (env->get_error_color().c_str(), errorcolor);
  gdk_color_parse (env->get_warning_color().c_str(), warningcolor);
  gtk_color_button_set_color (GTK_COLOR_BUTTON(error), errorcolor);
  gtk_color_button_set_color (GTK_COLOR_BUTTON(warning), warningcolor);
  gtk_font_button_set_font_name (GTK_FONT_BUTTON(fontbutton), fontnametemp.c_str());
}

GtkWidget* EnvironmentDialog::get_dialog()
{ return dialog;
}

// Initialize all of the objects and set all of the labels to be bold
void EnvironmentDialog::initialize_objects()
{
  PangoFontDescription *fd = pango_font_description_from_string("Sans Bold 10");
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/environment.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);

  dialog = glade_xml_get_widget(xml, "dialog");
  cancel = glade_xml_get_widget(xml, "button_cancel");
  ok = glade_xml_get_widget(xml, "button_ok");
  asave_minutes = glade_xml_get_widget(xml, "auto_save");
  tab_size = glade_xml_get_widget(xml, "tab_width");
  undo_actions = glade_xml_get_widget(xml, "undo_actions");
  autoindent = glade_xml_get_widget(xml, "auto_indent");
  bracket = glade_xml_get_widget(xml, "highlight_brackets");
  load_on_launch = glade_xml_get_widget(xml, "load_on_launch");
  replace_tabs = glade_xml_get_widget(xml, "insert_spaces");
  split_words = glade_xml_get_widget(xml, "split_words");
  text_wrap = glade_xml_get_widget(xml, "text_wrapping");
  right_margin = glade_xml_get_widget(xml, "display_margin");
  fontbutton = glade_xml_get_widget(xml, "fontbutton");
  error = glade_xml_get_widget(xml, "error_color");
  warning = glade_xml_get_widget(xml, "warning_color");
  label_tabstops = glade_xml_get_widget(xml, "label_tabstops");
  label_autoindent = glade_xml_get_widget(xml, "label_autoindent");
  label_filemanagement = glade_xml_get_widget(xml, "label_filemanagement");
  label_undo = glade_xml_get_widget(xml, "label_undo");
  label_text_wrap = glade_xml_get_widget(xml, "label_textwrapping");
  label_bracket = glade_xml_get_widget(xml, "label_bracketmatch");
  label_rightmargin = glade_xml_get_widget(xml, "label_rightmargin");
  label_font = glade_xml_get_widget(xml, "label_font");
  label_compiler = glade_xml_get_widget(xml, "label_compiler");
  
  // Set the labels to have a bold font
  gtk_widget_modify_font(label_autoindent, fd);
  gtk_widget_modify_font(label_bracket, fd);
  gtk_widget_modify_font(label_compiler, fd);
  gtk_widget_modify_font(label_filemanagement, fd);
  gtk_widget_modify_font(label_font, fd);
  gtk_widget_modify_font(label_tabstops, fd);
  gtk_widget_modify_font(label_text_wrap, fd);
  gtk_widget_modify_font(label_undo, fd);
  gtk_widget_modify_font(label_rightmargin, fd);
}

// Get the font size from the FontButton
string EnvironmentDialog::get_font_size()
{ 
  string font = gtk_font_button_get_font_name(GTK_FONT_BUTTON(fontbutton));
  
  while (font.find(" ", 0) != string::npos)
    font.erase(0, font.find(" ", 0)+1);
  
  return font;
}

// Get the font face from the FontButton
string EnvironmentDialog::get_font_face()
{ 
  string font = gtk_font_button_get_font_name(GTK_FONT_BUTTON(fontbutton));
  
  int loc = font.find(' ', 0);
  while (font.find(' ', loc+1) != string::npos)
    loc = font.find(' ', loc+1);
  
  font.erase(loc, font.length()-loc);
  return font;
}

// These functions allow the openldev class to get private members of this dialog
int EnvironmentDialog::get_autosave()
{ return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(asave_minutes));
}

int EnvironmentDialog::get_undo()
{ return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(undo_actions));
}

int EnvironmentDialog::get_tab_size()
{ return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tab_size));
}

bool EnvironmentDialog::get_load_on_launch()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(load_on_launch));
}

bool EnvironmentDialog::get_replace_tabs()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(replace_tabs));
}

bool EnvironmentDialog::get_auto_indent()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(autoindent));
}

bool EnvironmentDialog::get_wrap()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(text_wrap));
}

bool EnvironmentDialog::get_split()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(split_words));
}

bool EnvironmentDialog::get_brackets()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(bracket));
}

bool EnvironmentDialog::get_margin()
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(right_margin));
}

string get_color_string(GdkColor &color)
{
  char blue[3], green[3], red[3];
  snprintf(blue, sizeof(blue), "%X", color.blue);
  snprintf(red, sizeof(red), "%X", color.red);
  snprintf(green, sizeof(green), "%X", color.green);
  
  string clr = "#" + string(red);
  if (color.red == 0)
    clr += "0";
  clr += green;
  if (color.green == 0)
    clr += "0";
  clr += blue;
  if (color.blue == 0)
    clr += "0";
  
  return clr;
}

string EnvironmentDialog::get_error_color()
{ 
  GdkColor color;
  gtk_color_button_get_color(GTK_COLOR_BUTTON(error), &color);

  return get_color_string(color); 
}

string EnvironmentDialog::get_warning_color()
{ 
  GdkColor color;
  gtk_color_button_get_color(GTK_COLOR_BUTTON(warning), &color);
  
  return get_color_string(color);
}
