/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "find.h"

void search_all_files (vector<string>&, string, string, string, string);

// This function creates the find toolbar by initializing the widgets, creating
// the buttons and adding them to the horizontal box. The buttons are then
// connected to their signals
void OpenLDev::openldev_create_find_toolbar()
{
  findtoolbar = new FindToolbar();
  
  // Connect the buttons to their signals
  g_signal_connect(G_OBJECT(findtoolbar->get_close()), "clicked",
                   G_CALLBACK(on_close_toolbars), (gpointer)this);
  g_signal_connect(G_OBJECT(findtoolbar->get_find()), "clicked",
                   G_CALLBACK(on_find_next), (gpointer)this);
  g_signal_connect(G_OBJECT(findtoolbar->get_replace()), "clicked",
                   G_CALLBACK(on_find_and_replace), (gpointer)this);
}

// Toggle the visibility of the find toolbar. Also, this makes sure that if
// the Find toolbar is visible, the goto line toolbar is hidden
void openldev_find (GtkMenuItem *menuitem,
                    gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  if (GTK_WIDGET_VISIBLE(ptr->findtoolbar->get_hbox()))
    openldev_find_next(menuitem, data);
  else
  {
    gtk_widget_show(ptr->findtoolbar->get_hbox());
    gtk_widget_hide(ptr->gototoolbar->get_hbox());
  }
}

void openldev_find_next (GtkMenuItem *menuitem,
                         gpointer data) 
{
  OpenLDev *ptr = (OpenLDev*) data;
  on_find_next(ptr->findtoolbar->get_hbox(), data);
}

// Find the next iteration of the text the user specified in the buffer.
// If there is no selected text, search from the current cursor position.
// Otherwise, search from the selection bound. If the text is not found, wrap
// around the document once and then throw an error if it is still not found.
void on_find_next (GtkWidget *widget, 
                   gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  
  // Check if there was no text specified
  string entry_text = gtk_entry_get_text(GTK_ENTRY(ptr->findtoolbar->get_text()));
  if (!entry_text.empty())
  {
    if (ptr->findtoolbar->search_files_is_active())
    {
      gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->output->get_notebook()), 1);
      if (gtk_tree_view_get_model(GTK_TREE_VIEW(ptr->output->get_search_box())))
        gtk_tree_view_remove_column(GTK_TREE_VIEW(ptr->output->get_search_box()),
            gtk_tree_view_get_column(GTK_TREE_VIEW(ptr->output->get_search_box()), 0));                            
      
      vector<string> search_results;
      search_all_files(search_results, ptr->project->get_source_files(), 
                       ptr->project->get_header_files(), 
                       ptr->project->get_location(), entry_text);
      
      if (search_results.size() > 0)
      {
        GtkTreeModel *model;
	      GtkTreeIter iter;
        GtkCellRenderer *cell_renderer;
        GtkTreeViewColumn *column;
        
        model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING));
        
        cell_renderer = gtk_cell_renderer_text_new();
	      column = gtk_tree_view_column_new_with_attributes("Search Results", 
                          cell_renderer, "text", 0, "foreground", 1, NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(ptr->output->get_search_box()), column);
	
	      for (unsigned int i=0; i < search_results.size(); i++) 
        {
		      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, search_results.at(i).c_str(), 1, "Red", -1);
	      }
    
        gtk_tree_view_set_model(GTK_TREE_VIEW(ptr->output->get_search_box()), model);
	      g_object_unref(model);
        
        ptr->findtoolbar->show_error(false);
      }
      else
        ptr->findtoolbar->show_error(true);
    }
    else
    {
      GtkSourceBuffer *buffer = ptr->noteman->noteman_get_textbuffer();
      GtkTextIter iter, end, enditer;
      gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer), &enditer);
        
      // If there is no selected text, then search from the cursor position
      gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(buffer), &iter, 
        gtk_text_buffer_get_selection_bound(GTK_TEXT_BUFFER(buffer)));
      gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(buffer), &end, 
        gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer)));
      if (gtk_text_iter_compare(&end, &iter) > 0)
        iter = end;
  
      // If it is found, select the text
      if (gtk_text_iter_forward_search(&iter, 
          gtk_entry_get_text(GTK_ENTRY(ptr->findtoolbar->get_text())), 
          GTK_TEXT_SEARCH_VISIBLE_ONLY, &iter, &end, &enditer))
            ptr->openldev_mark_found_text(&iter, &end);
      // Throw an error if it cannot be found
      else 
      {
        gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &iter);
        if (gtk_text_iter_forward_search(&iter, entry_text.c_str(), 
            GTK_TEXT_SEARCH_VISIBLE_ONLY, &iter, &end, &enditer))
              ptr->openldev_mark_found_text(&iter, &end);
        else
          ptr->findtoolbar->show_error(true);
      }
    }
  }
}

// Mark the found text by selecting it. This function receives the beginning
// and end iterators of the found text. When this function is called, the error
// message and image on the find toolbar is hidden
void OpenLDev::openldev_mark_found_text (GtkTextIter *iter, 
                                         GtkTextIter *end)
{
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(noteman->noteman_get_textbuffer());
  gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_insert(buffer), iter);
  gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_selection_bound(buffer), end);
  noteman->noteman_scroll_to_mark();
  gtk_widget_grab_focus(noteman->noteman_get_textview());
      
  findtoolbar->show_error(false);
}

// Search all of the files in a project for the text you want to find. This
// function receives the vector of search results to edit by reference, and
// strings with the source and header files, the location of the project
// directory and the text to search for in the files.
void search_all_files (vector<string> &search_results, 
                       string sources,
                       string headers, 
                       string location, 
                       string find_text)
{
  string files = sources + headers;
  
  // while there are files left to search
  while (files.length() > 0)
  {
    // Create the file string & erase it from the list
    int loc = files.find('/', 0), line = 1;
    string text = location + "src/" + files.substr(0, loc), file = files.substr(0, loc);
    files.erase(0, loc+1);
    
    // Open the file & get it line by line
    ifstream fin(text.c_str());
    while (getline(fin, text))
    {
      // If the text is found, add a string to the search results vector in
      // the form - filename:line: Line Content
      if (text.find(find_text, 0) != string::npos)
      {
        gchar buff[5];
        snprintf(buff, sizeof(buff), "%d", line);
        search_results.push_back(string(file + ":" + buff + ": " + text));
      }
      
      line++;
    }
    
    fin.close();
  }
}

// Catch the search box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and then the 
// file containing the search text is opened. The cursor is then moved line of 
// the search result
bool on_search_box_selected (GtkWidget *widget,
                             GdkEventButton *event,
                             gpointer data)
{
  OpenLDev *ptr = (OpenLDev*)data;
  
  if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      string file;
      gchar *output;
      int linenumber;
      gtk_tree_model_get(model, &iter, 0, &output, -1);
      
      // Open the file the error was found in
      file = string(output);
      linenumber = get_file_name_and_line(file) - 1;
      ptr->openldev_open_file(ptr->project->get_location() + "src/" + file);
        
      // Move the cursor to the line of the error & scroll to the cursor
      GtkTextIter iter;
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman->noteman_get_textbuffer());
      
      gtk_text_buffer_get_iter_at_line(buffer, &iter, linenumber);
      gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_insert(buffer), &iter);
      gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
      ptr->noteman->noteman_scroll_to_mark();
      gtk_widget_grab_focus(ptr->noteman->noteman_get_textview());
    }
    
    return true;
  }
  
  return false;
}

// Replace the specified text with the text specified by the user in the input
// dialog. By choosing the checkbox to search in all files, you can replace the
// text in all of the files.
void on_find_and_replace (GtkWidget *widget, 
                          gpointer data)
{
  OpenLDev *ptr = (OpenLDev*) data;
  int count = 0;
  bool process_replace = true;
  string replace_text;
  GtkWidget *item = gtk_menu_item_new();
  openldev_find_next(GTK_MENU_ITEM(item), data);
  
  // Loop until the text is no longer found
  while (ptr->noteman->noteman_get_insert_offset() != ptr->noteman->noteman_get_selection_offset())
  {
    // Get the text to replace with from the user
    if (count == 0)
    {
      InputDialog *dialog;
      dialog = new InputDialog ("Replace With: ", "Replace Text");
      int result = gtk_dialog_run(GTK_DIALOG(dialog->get_dialog()));
     
      // Either say to stop replace or get the text to replace with
      switch (result)
      {
        case GTK_RESPONSE_OK:
          replace_text = dialog->get_entry_text();
          break;
        default:
          process_replace = false;
          break;
      }
      gtk_widget_hide(dialog->get_dialog());
    }
    
    // Cancel the replacing
    if (!process_replace)
      break;
    
    // Erase the found text & replace it
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ptr->noteman->noteman_get_textbuffer());
    GtkTextIter insert, select;
    
    gtk_text_buffer_get_iter_at_mark(buffer, &insert,
                                     gtk_text_buffer_get_insert(buffer));
    gtk_text_buffer_get_iter_at_mark(buffer, &select,
                                     gtk_text_buffer_get_selection_bound(buffer));
    gtk_text_buffer_delete(buffer, &insert, &select);
    gtk_text_buffer_insert_at_cursor(buffer, replace_text.c_str(), -1);
    
    // Refresh the cursor and find the next iteration
    gtk_text_buffer_get_iter_at_mark(buffer, &select,
                                     gtk_text_buffer_get_selection_bound(buffer));
    gtk_text_buffer_move_mark_by_name(buffer, "insert", &select);
    
    openldev_find_next (GTK_MENU_ITEM(item), data);
    count++;
  }
  
  // Hide the error if there was text found
  if (count > 0)
    ptr->findtoolbar->show_error(false);
}
