/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "outputmanager.h"

OutputManager::OutputManager()
{
  search = new OutputSearch();
  build = new OutputBuild();
  terminal = new OutputTerminal();
  notebook = gtk_notebook_new();
  
  compiler = gtk_hbox_new(false, 0);
  compiler_label = gtk_label_new("Compiler");
  compiler_image = gtk_image_new_from_stock("gtk-execute", GTK_ICON_SIZE_MENU);
  findtab = gtk_hbox_new(false, 0);
  findtab_label = gtk_label_new("Search");
  findtab_image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_MENU);
  terminaltab = gtk_hbox_new(false, 0);
  terminaltab_label = gtk_label_new("Terminal");
  terminaltab_image = gtk_image_new_from_stock("gtk-justify-left", GTK_ICON_SIZE_MENU);
  
  gtk_box_set_spacing(GTK_BOX(compiler), 5);
  gtk_box_pack_start(GTK_BOX(compiler), compiler_image, false, false, 0);
  gtk_box_pack_start(GTK_BOX(compiler), compiler_label, false, false, 0);
  gtk_box_set_spacing(GTK_BOX(findtab), 5);
  gtk_box_pack_start(GTK_BOX(findtab), findtab_image, false, false, 0);
  gtk_box_pack_start(GTK_BOX(findtab), findtab_label, false, false, 0);
  gtk_box_set_spacing(GTK_BOX(terminaltab), 5);
  gtk_box_pack_start(GTK_BOX(terminaltab), terminaltab_image, false, false, 0);
  gtk_box_pack_start(GTK_BOX(terminaltab), terminaltab_label, false, false, 0);
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           build->get_swin(),
                           compiler);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           search->get_swin(),
                           findtab);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           terminal->get_hbox(),
                           terminaltab);  
  
  gtk_widget_show(compiler);
  gtk_widget_show(compiler_image);
  gtk_widget_show(compiler_label);
  gtk_widget_show(findtab);
  gtk_widget_show(findtab_image);
  gtk_widget_show(findtab_label);
  gtk_widget_show(terminaltab);
  gtk_widget_show(terminaltab_image);
  gtk_widget_show(terminaltab_label);
  gtk_widget_show(notebook);
}

void OutputManager::run_terminal_command (string command)
{ 
  terminal->run_command(command);
  grab_terminal_focus();
}

void OutputManager::grab_terminal_focus()
{ terminal->grab_focus();
}

GtkWidget* OutputManager::get_build_box()
{ return build->get_build_box();
}

GtkWidget* OutputManager::get_search_box()
{ return search->get_search_box();
}

GtkWidget* OutputManager::get_build_swin()
{ return build->get_swin();
}

GtkWidget* OutputManager::get_search_swin()
{ return search->get_swin();
}

GtkWidget* OutputManager::get_notebook()
{ return notebook;
}

GtkWidget* OutputManager::get_terminal()
{ return terminal->get_terminal();
}

OutputBuild* OutputManager::get_build()
{ return build;
}
